# -*- coding: utf-8 -*-

import logging
import sys
import itertools
import numpy as np

from passwordcard import passwordcard

logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.INFO)

class Card:
    """ Represents a password card """
    def __init__(self, args):
        # Store card as matrix
        self._m = None
        # Stores password streams generated from the card
        self._args = vars(args)
        self._seed = self._args.get('seed')
        self._header = None

    @staticmethod
    def _validate_matrix(m):
        """ Validates custom cards supplied by the user """
        try:
            matrix = np.matrix(m)
        except TypeError:
            logging.error("Invalid password card format, please try again")
            sys.exit(2)
        # Check whether each line of matrix is equal
        for i in range(0, len(m)-1):
            if not len(m[i]) == len(m[i+1]):
                logging.error("Invalid password card format, please try again")
                sys.exit(2)
        # Return matrix if its format is a-okay
        return matrix

    # --------------------------------------------------------
    #
    # Stream generators
    #
    # --------------------------------------------------------

    def _left_to_right(self):
        """ If the password from the card is read from left to right """
        data = self._m.getA1().flatten()
        return data.tolist()

    def _right_to_left(self):
        """ If the password from the card is read from right to left """
        data = self._left_to_right()
        data.reverse()
        return data

    def _top_to_down(self):
        """ If the password from the card is read from top to down """
        data = self._m.getT().getA1().flatten()
        return data.tolist()

    def _bottom_to_top(self):
        """ If the password from the card is read from bottom to up """
        data = self._top_to_down()
        data.reverse()
        return data

    def _zig_zag(self):
        """ If the password from the card is read in zig-zag directions """
        rows = self._m.getA().tolist()
        for i in range(len(rows)):
            # Reverse order on every second line
            if i % 2 <> 0:
                rows[i].reverse()
        # Flatten list
        data = list(itertools.chain.from_iterable(rows))
        return data

    def _zig_zag_reverse(self):
        """ If the password from the card is read in reverse zig-zag directions """
        rows = self._m.getA().tolist()
        rows.reverse()
        for i in range(len(rows)):
            # Reverse order on every second line
            if i % 2 == 0:
                rows[i].reverse()
        # Flatten list
        data = list(itertools.chain.from_iterable(rows))
        return data

    def _diagonal(self):
        """ If the password from the card is read diagonally """
        diagonals = []
        for i in range(self.rows * -1, self.columns):
            diagonals.append(self._m.diagonal(offset=i).tolist()[0])
        data = list(itertools.chain.from_iterable(diagonals))
        return data

    def _angled(self):
        """ If the password from the card is read in a rotated 'L' shape """
        data = []
        for i in range(0, self.rows-1):
            # Select a row
            stream = self._m[i].tolist()[0]
            # Select last elements from each row below
            for j in range(i+1, self.rows):
                stream.append(self._m[j].tolist()[0][-1])
            data.append(stream)
        return data

    # --------------------------------------------------------

    def _generate_data_streams(self):
        """ Adds appropriate character streams based on the selected card reading strategies """
        streams = []
        if self._args.get('left_to_right') or self._args.get('all'):
            streams.append(self._left_to_right())
        if self._args.get('right_to_left') or self._args.get('all'):
            streams.append(self._right_to_left())
        if self._args.get('top_down') or self._args.get('all'):
            streams.append(self._top_to_down())
        if self._args.get('bottom_up') or self._args.get('all'):
            streams.append(self._bottom_to_top())
        if self._args.get('zig_zag') or self._args.get('all'):
            streams.append(self._zig_zag())
        if self._args.get('zig_zag_reverse') or self._args.get('all'):
            streams.append(self._zig_zag_reverse())
        if self._args.get('diagonal') or self._args.get('all'):
            streams.append(self._diagonal())
        if self._args.get('angled') or self._args.get('all'):
            data = self._angled()
            for i in range(0, len(data)):
                streams.append(data[i])
        return streams

    @staticmethod
    def _passwords_generator(streams, pwlen):
        """ Generator that dumps the passwords for each card read strategy """
        # Iterate through streams (strategies)
        for stream in streams:
            counter = 0
            # Generate passwords with certain length from the card
            while counter+pwlen < len(stream)+1:
                result = stream[counter:counter+pwlen]
                # Return result back
                yield ''.join(result)
                counter += 1

    def generate_password_card(self, digits=False, symbols=False):
        """ Generate password card as on http://passwordcard.org """
        m = []
        # Generate card
        header, card = passwordcard.generate_card(self._seed, digits=digits, symbols=symbols)
        # Add lists to a list and generate matrix from that with NumPy
        for row in card:
            m.append(list(row))
        # Validate and generate matrix with NumPy
        self._m = self._validate_matrix(m)
        self._header = header
        # Save card dimensions
        return self._m

    def generate_custom_card(self, m):
        """ Generates custom password card """
        # Validate and generate matrix with NumPy
        self._m = self._validate_matrix(m)
        # Save card dimensions
        return self._m

    @property
    def rows(self):
        """ Get number of password card rows """
        return self._m.shape[0]

    @property
    def columns(self):
        """ Get number of password card columns """
        return self._m.shape[1]

    @property
    def print_card(self):
        """ Generates fancy password card box """
        output = []
        # Assign text header
        try:
            # Display seed in case of passwordcard.org cards
            header = " %s " % self._seed
        except AttributeError:
            # Display generic title for custom cards
            header = " PASSWORD CARD "
        # Verify wether card is wide enough to display header text
        if len(header)+2 > self.columns:
            # Box is too narrow, do not add text header
            top = "+-" + ("-" * self.columns) + "-+\r"
        else:
            # Add text header
            top = "+-%s-+\r" % header.center(self.columns, "-")
        output.append(top)
        # Add empty line between top and body
        empty_line = "| " + (" " * self.columns) + " |"
        output.append(empty_line)
        # Add header of symbols if card is from passwordcard.org
        if self._header:
            row = "| %s |" % self._header.center(self.columns)
            output.append(row)
            output.append(empty_line)
        # Add password card body
        for i in range(len(self._m)):
            row = ''.join(self._m.tolist()[i])
            row = "| %s |" % row.center(self.columns)
            output.append(row)
        # Add empty line between body and bottom
        output.append(empty_line)
        # Add bottom with password card dimensions
        dimensions = " %sx%s " % (self.rows, self.columns)
        bottom = "+%s+\r" % dimensions.center(self.columns+2, "-")
        output.append(bottom)
        output.append("\r")
        # Return full box with password card as the output
        return '\r\n'.join(output)

    @property
    def passwords(self):
        """ Dumps passwords from the password card """
        # Generate streams based on the card read strategies
        streams = self._generate_data_streams()
        # Iterate through all password lengths between minimum and maximum
        for pwlen in range(int(self._args.get('minlen')), int(self._args.get('maxlen'))+1):
            # Dump password on the screen constructed by the generator above
            for password in self._passwords_generator(streams, pwlen):
                yield password



__description__ = 'Wordlist generator based on password cards'
__title__ = 'munchkin'
__version_info__ = ('0', '2', '5')
__version__ = '.'.join(__version_info__)
__author__ = 'Gabor Szathmari'
__credits__ = ['Gabor Szathmari']
__maintainer__ = 'Gabor Szathmari'
__email__ = 'gszathmari@gmail.com'
__status__ = 'alpha'
__license__ = 'MIT'
__copyright__ = '(c) 2015 Gabor Szathmari'
__website__ = 'http://github.com/gszathmari/munchkin'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import munchkin

package_folder = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, package_folder)

if __name__ == '__main__':
    sys.exit(munchkin.main(sys.argv[1:]))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# passwordcard - A passwordcard generator compatible with passwordcard.org
#
# Copyright © 2012 Nicolas Dandrimont <nicolas.dandrimont@crans.org>
#
#   This file is part of PasswordCard.
#
#   PasswordCard is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from .passwordcard import *



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# passwordcard - A passwordcard generator compatible with passwordcard.org
#
# Copyright © 2012 Nicolas Dandrimont <nicolas.dandrimont@crans.org>
#
#   This file is part of PasswordCard.
#
#   PasswordCard is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import javarandom

CHARSETS = {
    'original.digits': u"0123456789",
    'original.alphanumeric': u"23456789abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ",
    'original.alphanumeric_with_symbols': u"23456789abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ@#$%&*<>?€+{}[]()/\\"
    }

HEADERS = {
    'original': u"■□▲△○●★☂☀☁☹☺♠♣♥♦♫€¥£$!?¡¿⊙◐◩�",
    }

# Construct the appropriate character sets
def generate_character_sets(symbols, digits):
    characters = {}
    # Select characters for the card
    if symbols and not digits:
        characters['top_odd'] = list(CHARSETS['original.alphanumeric'])
        characters['top_even'] = list(CHARSETS['original.alphanumeric_with_symbols'])
        characters['bottom_odd'] = list(CHARSETS['original.alphanumeric'])
        characters['bottom_even'] = list(CHARSETS['original.alphanumeric_with_symbols'])
    elif symbols and digits:
        characters['top_odd'] = list(CHARSETS['original.alphanumeric'])
        characters['top_even'] = list(CHARSETS['original.alphanumeric_with_symbols'])
        characters['bottom_odd'] = list(CHARSETS['original.digits'])
        characters['bottom_even'] = list(CHARSETS['original.digits'])
    elif not symbols and digits:
        characters['top_odd'] = list(CHARSETS['original.alphanumeric'])
        characters['top_even'] = list(CHARSETS['original.alphanumeric'])
        characters['bottom_odd'] = list(CHARSETS['original.digits'])
        characters['bottom_even'] = list(CHARSETS['original.digits'])
    elif not symbols and not digits:
        characters['top_odd'] = list(CHARSETS['original.alphanumeric'])
        characters['top_even'] = list(CHARSETS['original.alphanumeric'])
        characters['bottom_odd'] = list(CHARSETS['original.alphanumeric'])
        characters['bottom_even'] = list(CHARSETS['original.alphanumeric'])
    else:
        raise Exception("Cannot choose charset for password card")
    return characters

def generate_card(seed, width=29, height=8, symbols=False, digits=False):
    characters = generate_character_sets(symbols, digits)

    seed = int("0x%s" % seed, 16)

    """Generate a password card with the given parameters"""
    rng = javarandom.JavaRandom(seed)

    header = list(HEADERS['original'])

    rng.shuffle(header)

    contents = []

    midheight = 1 + (height/2)

    for i in range(1, midheight):
        line = []
        for j in range(width):
            # Even columns
            if j % 2 == 0:
                line.append(characters['top_even'][rng.next_int(len(characters['top_even']))])
            # Odd columns
            else:
                line.append(characters['top_odd'][rng.next_int(len(characters['top_odd']))])
        contents.append(u''.join(line))
    for j in range(midheight, height+1):
        line = []
        for j in range(width):
            # Even columns
            if j % 2 == 0:
                line.append(characters['bottom_even'][rng.next_int(len(characters['bottom_even']))])
            # Odd columns
            else:
                line.append(characters['bottom_odd'][rng.next_int(len(characters['bottom_odd']))])
        contents.append(u''.join(line))

    return u''.join(header), contents



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# javarandom - Java-compatible random number generator
#
# Copyright © 2012 Nicolas Dandrimont <nicolas.dandrimont@crans.org>
#
#   This file is part of PasswordCard.
#
#   PasswordCard is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import threading
import time

class JavaRandom(object):
    """A partial implementation of a java-compatible random number generator"""
    def __init__(self, seed = None):
        if seed is None:
            seed = int(time.time() * 1000)

        self.seed = seed
        self.lock = threading.Lock()

    @property
    def seed(self):
        return self._seed

    @seed.setter
    def seed(self, seed):
        self._seed = (seed ^ 0x5DEECE66D) & ((1 << 48) - 1)

    def next(self, bits):
        """Return `bits` random bits as a signed 32 bits integer"""
        with self.lock:
            self._seed = (self._seed * 0x5DEECE66D + 0xB) & ((1 << 48) - 1)

            ret = self._seed >> (48 - bits)

            if ret & (1 << 31):
                ret -= (1 << 32)

            return ret

    def next_int(self, maxint = None):
        """Return an integer evenly distributed between 0 and maxint-1"""
        if maxint is None:
            return self.next(32)
        if maxint <= 0 or maxint > (1 << 31) - 1:
            raise ValueError("maxint needs to be in range 1 - 2^31-1")

        # if maxint is a power of two, just return the right number of bits (MSB of self.next)
        if (maxint & -maxint) == maxint:
            return (maxint * self.next(31)) >> 31

        bits = self.next(31)
        val = bits % maxint

        # We need to emulate the 32-bit int here too
        #     (bits - val + maxint - 1) < 0
        while (bits - val + maxint - 1) & (1 << 31):
            bits = self.next(31)
            val = bits % maxint

        return val

    def shuffle(self, list):
        """Shuffle the list given in the argument, using the algorithm from java.collections"""
        for i in reversed(range(2, len(list)+1)):
            rnd = self.next_int(i)
            list[i-1], list[rnd] = list[rnd], list[i-1]




from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="munchkin",
    version="0.2.5",
    author="Gabor Szathmari",
    author_email="gszathmari@gmail.com",
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    include_package_data=True,
    url="https://github.com/gszathmari/munchkin",
    license="MIT",
    platforms = ["Linux"],
    description="Wordlist generator based on password cards",
    long_description=long_description,
    entry_points={
        'console_scripts': [
            'munchkin=munchkin:main',
        ],
    },
    install_requires=["numpy==1.9.3"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Topic :: Security',
        'Intended Audience :: Other Audience',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='security password',
)



# -*- coding: utf-8 -*-

import argparse
import random
import string
import subprocess
import sys
import os
import logging

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.insert(0, os.path.join(os.path.dirname(os.path.abspath(__file__)), 'lib'))

from __about__ import __version__, __author__
from card import Card

logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.INFO)

def password_dumper(args, card):
    """ Displays card and passwords or dumps them to file """
    # If this is a real terminal, display password card on screen
    if sys.stdout.isatty():
        logging.info("Printing password card")
        print("\n%s" % card.print_card)
    # Dump passwords to file if this option was selected
    if args.file:
        f = args.file
        logging.info("Dumping to file: %s ... " % f.name)
        for password in card.passwords:
            f.write("%s\n" % password)
        f.close()
        logging.info("Write completed")
    # Just display the passwords if user pipes this command
    else:
        for password in card.passwords:
            print(password)

def generate_card_pcard(args, card):
    """ Generate card similar to ones on http://passwordcard.org """
    card.generate_password_card(symbols=args.symbols, digits=args.digits)
    password_dumper(args, card)

def generate_card_custom(args, card):
    """ Generate custom, user supplied password card """
    data = []
    print("Please copy-paste password card below (press ENTER twice when done):\r\n")
    # Read card from terminal
    while True:
        input_str = raw_input()
        # Finish reading when user leaves line empty
        if input_str == "":
            break
        else:
            row = list(input_str)
            data.append(row)
    card.generate_custom_card(data)
    password_dumper(args, card)

def menu():
    """ Contains input arguments and launches controller """
    version_info="Munchkin wordlist generator version %s by %s" % (__version__, __author__)
    parser = argparse.ArgumentParser()

    parser.add_argument('-v', '--version', help='version information', action='version', version=version_info)

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument('-f', '--file', type=argparse.FileType('wb', 0), metavar='name', help='dump passwords to file')

    pwlen_options = parent_parser.add_argument_group('password lengths')
    pwlen_options.add_argument('--minlen', help='minimum password length (default: 6)', default=6, metavar='num')
    pwlen_options.add_argument('--maxlen', help='maximum password length (default: 12)', default=12, metavar='num')

    wordlist_options = parent_parser.add_argument_group('wordlist generator algorithms')
    wordlist_options.add_argument('-l', '--left-to-right', help='read card from top left to bottom right', action='store_true')
    wordlist_options.add_argument('-r', '--right-to-left', help='read card from bottom right to top left', action='store_true')
    wordlist_options.add_argument('-t', '--top-down', help='read card from top left to bottom left', action='store_true')
    wordlist_options.add_argument('-b', '--bottom-up', help='read card from bottom right to top right', action='store_true')
    wordlist_options.add_argument('-z', '--zig-zag', help='read card zig-zag from top left', action='store_true')
    wordlist_options.add_argument('-x', '--zig-zag-rev', help='read card zig-zag from bottom right', action='store_true')
    wordlist_options.add_argument('-d', '--diagonal', help='read card diagonally', action='store_true')
    wordlist_options.add_argument('-A', '--angled', help='read card at right-angle directions', action='store_true')
    wordlist_options.add_argument('-a', '--all', help='use each read strategy from above', action='store_true')

    subparsers = parser.add_subparsers(title='subcommands', description='valid subcommands', help='Generate wordlist based on cards from:')
    parser_pc = subparsers.add_parser('pcard', help="http://passwordcard.org", parents=[parent_parser])
    parser_custom = subparsers.add_parser('custom', help="your terminal", parents=[parent_parser])

    passwordcard_options = parser_pc.add_argument_group('passwordcard options')
    passwordcard_options.add_argument('-s', '--seed', help='card number (e.g. 7eb3fbfa560d1d1e)', required=True, metavar='str')
    passwordcard_options.add_argument('--symbols', help='include symbols', action='store_true')
    passwordcard_options.add_argument('--digits', help='incude digits', action='store_true')

    # Set controller functions
    passwordcard_options.set_defaults(func=generate_card_pcard)
    parser_custom.set_defaults(func=generate_card_custom)

    args = parser.parse_args()

    controller(args)

def controller(args):
    """ Verifies arguments and dispatches appropriate functions """
    # Verifies whether at least one strategy was selected
    if (args.left_to_right or
        args.right_to_left or
        args.top_down or
        args.bottom_up or
        args.zig_zag or
        args.zig_zag_rev or
        args.diagonal or
        args.angled or
        args.all) is False:
        logging.error("Please select at least one algorithm (hint: -l/-r/-t/-b/-z/-x/-d/-A/--all)\r\n")
        sys.exit(3)
    else:
        # Create card and start processing
        card = Card(args)
        args.func(args, card)

# Displays banner and program version
def banner():
    """ Generates fancy banner """
    banner = """
 _______           _        _______           _       _________ _
(       )|\     /|( (    /|(  ____ \|\     /|| \    /\\__   __/( (    /|
| () () || )   ( ||  \  ( || (    \/| )   ( ||  \  / /   ) (   |  \  ( |
| || || || |   | ||   \ | || |      | (___) ||  (_/ /    | |   |   \ | |
| |(_)| || |   | || (\ \) || |      |  ___  ||   _ (     | |   | (\ \) |
| |   | || |   | || | \   || |      | (   ) ||  ( \ \    | |   | | \   |
| )   ( || (___) || )  \  || (____/\| )   ( ||  /  \ \___) (___| )  \  |
|/     \|(_______)|/    )_)(_______/|/     \||_/    \/\_______/|/    )_)\r\n"""
    print("Munchkin {0}\r".format(__version__))
    print(banner)

def main(argv=None):
    """ Main program entry point """
    # Only display the banner if the output is terminal
    if sys.stdout.isatty():
        banner()
    menu()
